package sun.tracing.dtrace;

import java.lang.reflect.InvocationHandler;
import com.sun.tracing.dtrace.DependencyClass;
import com.sun.tracing.dtrace.Attributes;
import com.sun.tracing.dtrace.StabilityLevel;
import java.lang.annotation.Annotation;
import com.sun.tracing.dtrace.FunctionName;
import com.sun.tracing.ProbeName;
import com.sun.tracing.dtrace.ModuleName;
import sun.misc.ProxyGenerator;
import java.lang.reflect.Modifier;
import com.sun.tracing.Provider;
import sun.tracing.ProbeSkeleton;
import java.lang.reflect.Method;
import sun.tracing.ProviderSkeleton;

class DTraceProvider extends ProviderSkeleton
{
    private Activation activation;
    private Object proxy;
    private static final Class[] constructorParams;
    private final String proxyClassNamePrefix = "$DTraceTracingProxy";
    static final String DEFAULT_MODULE = "java_tracing";
    static final String DEFAULT_FUNCTION = "unspecified";
    private static long nextUniqueNumber;
    
    private static synchronized long getUniqueNumber() {
        return DTraceProvider.nextUniqueNumber++;
    }
    
    protected ProbeSkeleton createProbe(Method method) {
        return new DTraceProbe(this.proxy, method);
    }
    
    DTraceProvider(Class<? extends Provider> clazz) {
        super(clazz);
    }
    
    void setProxy(Object proxy) {
        this.proxy = proxy;
    }
    
    void setActivation(Activation activation) {
        this.activation = activation;
    }
    
    public void dispose() {
        if (this.activation != null) {
            this.activation.disposeProvider(this);
            this.activation = null;
        }
        super.dispose();
    }
    
    public <T extends Provider> T newProxyInstance() {
        long uniqueNumber = getUniqueNumber();
        String str = "";
        if (!Modifier.isPublic(this.providerType.getModifiers())) {
            String name = this.providerType.getName();
            int lastIndex = name.lastIndexOf(46);
            str = ((lastIndex == -1) ? "" : name.substring(0, lastIndex + 1));
        }
        String string = str + "$DTraceTracingProxy" + uniqueNumber;
        byte[] generateProxyClass = ProxyGenerator.generateProxyClass(string, new Class[] { this.providerType });
        Class<Object> defineClass;
        try {
            defineClass = JVM.defineClass(this.providerType.getClassLoader(), string, generateProxyClass, 0, generateProxyClass.length);
        }
        catch (ClassFormatError classFormatError) {
            throw new IllegalArgumentException(classFormatError.toString());
        }
        try {
            return (T)defineClass.getConstructor(DTraceProvider.constructorParams).newInstance(new Object[] { this });
        }
        catch (ReflectiveOperationException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    protected void triggerProbe(Method method, Object[] array) {
        assert false : "This method should have been overridden by the JVM";
        return;
    }
    
    public String getProviderName() {
        return super.getProviderName();
    }
    
    String getModuleName() {
        return getAnnotationString(this.providerType, ModuleName.class, "java_tracing");
    }
    
    static String getProbeName(Method method) {
        return getAnnotationString(method, ProbeName.class, method.getName());
    }
    
    static String getFunctionName(Method method) {
        return getAnnotationString(method, FunctionName.class, "unspecified");
    }
    
    DTraceProbe[] getProbes() {
        return (DTraceProbe[])this.probes.values().toArray(new DTraceProbe[0]);
    }
    
    StabilityLevel getNameStabilityFor(Class<? extends Annotation> clazz) {
        Attributes attributes = (Attributes)getAnnotationValue(this.providerType, clazz, "value", null);
        if (attributes == null) {
            return StabilityLevel.PRIVATE;
        }
        return attributes.name();
    }
    
    StabilityLevel getDataStabilityFor(Class<? extends Annotation> clazz) {
        Attributes attributes = (Attributes)getAnnotationValue(this.providerType, clazz, "value", null);
        if (attributes == null) {
            return StabilityLevel.PRIVATE;
        }
        return attributes.data();
    }
    
    DependencyClass getDependencyClassFor(Class<? extends Annotation> clazz) {
        Attributes attributes = (Attributes)getAnnotationValue(this.providerType, clazz, "value", null);
        if (attributes == null) {
            return DependencyClass.UNKNOWN;
        }
        return attributes.dependency();
    }
    
    static {
        constructorParams = new Class[] { InvocationHandler.class };
        DTraceProvider.nextUniqueNumber = 0L;
    }
}
